@@ -35,6 +35,24 @@ except NameError:
         def __length_hint__(self):
             return self.remaining
 
+try:
+    any = any
+except NameError:
+    def any(iterable):
+        for x in iterable:
+            if x:
+                return True
+        return False
+
+try:
+    all = all
+except NameError:
+    def all(iterable):
+        for x in iterable:
+            if not x:
+                return False
+        return True
+
 try:
     sorted = sorted
 except NameError:
@@ -67,10 +85,10 @@ except NameError:
 try:
     set, frozenset = set, frozenset
 except NameError:
-    from sets import set, frozenset 
+    from sets import set, frozenset
 
 # pass through
-enumerate = enumerate 
+enumerate = enumerate
 
 try:
     BaseException = BaseException
@@ -87,24 +105,39 @@ except NameError:
         pass
     GeneratorExit.__module__ = 'exceptions'
 
+_sysex = (KeyboardInterrupt, SystemExit, MemoryError, GeneratorExit)
+
+try:
+    callable = callable
+except NameError:
+    def callable(obj):
+        return hasattr(obj, "__call__")
+
 if sys.version_info >= (3, 0):
     exec ("print_ = print ; exec_=exec")
     import builtins
 
-    # some backward compatibility helpers 
-    _basestring = str 
-    def _totext(obj, encoding=None):
+    # some backward compatibility helpers
+    _basestring = str
+    def _totext(obj, encoding=None, errors=None):
         if isinstance(obj, bytes):
-            obj = obj.decode(encoding)
+            if errors is None:
+                obj = obj.decode(encoding)
+            else:
+                obj = obj.decode(encoding, errors)
         elif not isinstance(obj, str):
             obj = str(obj)
         return obj
 
-    def _isbytes(x): 
+    def _isbytes(x):
         return isinstance(x, bytes)
-    def _istext(x): 
+    def _istext(x):
         return isinstance(x, str)
 
+    text = str
+    bytes = bytes
+
+
     def _getimself(function):
         return getattr(function, '__self__', None)
 
@@ -122,7 +155,7 @@ if sys.version_info >= (3, 0):
             del back
         elif locs is None:
             locs = globs
-        fp = open(fn, "rb")
+        fp = open(fn, "r")
         try:
             source = fp.read()
         finally:
@@ -130,18 +163,17 @@ if sys.version_info >= (3, 0):
         co = compile(source, fn, "exec", dont_inherit=True)
         exec_(co, globs, locs)
 
-    def callable(obj):
-        return hasattr(obj, "__call__")
-
 else:
     import __builtin__ as builtins
-    _totext = unicode 
+    _totext = unicode
     _basestring = basestring
+    text = unicode
+    bytes = str
     execfile = execfile
     callable = callable
-    def _isbytes(x): 
+    def _isbytes(x):
         return isinstance(x, str)
-    def _istext(x): 
+    def _istext(x):
         return isinstance(x, unicode)
 
     def _getimself(function):
@@ -157,7 +189,7 @@ else:
             return getattr(function, "func_code", None)
 
     def print_(*args, **kwargs):
-        """ minimal backport of py3k print statement. """ 
+        """ minimal backport of py3k print statement. """
         sep = ' '
         if 'sep' in kwargs:
             sep = kwargs.pop('sep')
@@ -177,22 +209,22 @@ else:
         file.write(end)
 
     def exec_(obj, globals=None, locals=None):
-        """ minimal backport of py3k exec statement. """ 
+        """ minimal backport of py3k exec statement. """
         __tracebackhide__ = True
-        if globals is None: 
+        if globals is None:
             frame = sys._getframe(1)
-            globals = frame.f_globals 
+            globals = frame.f_globals
             if locals is None:
                 locals = frame.f_locals
         elif locals is None:
             locals = globals
-        exec2(obj, globals, locals) 
+        exec2(obj, globals, locals)
 
 if sys.version_info >= (3,0):
     def _reraise(cls, val, tb):
         __tracebackhide__ = True
         assert hasattr(val, '__traceback__')
-        raise val
+        raise cls.with_traceback(val, tb)
 else:
     exec ("""
 def _reraise(cls, val, tb):
@@ -200,15 +232,17 @@ def _reraise(cls, val, tb):
     raise cls, val, tb
 def exec2(obj, globals, locals):
     __tracebackhide__ = True
-    exec obj in globals, locals 
+    exec obj in globals, locals
 """)
 
 def _tryimport(*names):
-    """ return the first successfully imported module. """ 
+    """ return the first successfully imported module. """
     assert names
     for name in names:
         try:
-            return __import__(name, None, None, '__doc__')
+            __import__(name)
         except ImportError:
             excinfo = sys.exc_info()
+        else:
+            return sys.modules[name]
     _reraise(*excinfo)
