@@ -1,16 +1,17 @@
 """
 local path implementation.
 """
+from __future__ import with_statement
+
 from contextlib import contextmanager
 import sys, os, re, atexit, io
 import py
 from py._path import common
+from py._path.common import iswin32
 from stat import S_ISLNK, S_ISDIR, S_ISREG
 
 from os.path import abspath, normpath, isabs, exists, isdir, isfile, islink
 
-iswin32 = sys.platform == "win32" or (getattr(os, '_name', False) == 'nt')
-
 if sys.version_info > (3,0):
     def map_as_list(func, iter):
         return list(map(func, iter))
@@ -151,7 +152,7 @@ class LocalPath(FSBase):
         elif isinstance(path, py.builtin._basestring):
             if expanduser:
                 path = os.path.expanduser(path)
-            self.strpath = abspath(normpath(path))
+            self.strpath = abspath(path)
         else:
             raise ValueError("can only pass None, Path instances "
                              "or non-empty strings to LocalPath")
